'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
''' 


'''

Notes about the current design:

The level is 10000 pixels high.  The background is mostly pregenerated.

'''

import background
import birds
import data
import os
import pygame
import sys
import utilities
from pygame.locals import *

# Max velocity for going up.
MAX_UP_SPEED = 10
# Accelerate for two seconds to max speed.
UP_SPEED_INC = MAX_UP_SPEED/float(2*60)
UP_SPEED_DEC = MAX_UP_SPEED/float(2*60)

def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer:
        return NoneSound()
    fullname = os.path.join('data', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print "Cannot load sound:", wav
        raise SystemExit, message
    return sound

class Balloon(pygame.sprite.Sprite):
    """Balloons attached to the balloon guy"""
    def __init__(self, balloon_guy, x_offset, y_offset):
        pygame.sprite.Sprite.__init__(self) #call Sprite initializer
        self.image, self.rect = utilities.load_image('balloon1.bmp', -1)
        self.balloon_guy = balloon_guy
        self.x_offset = x_offset
        self.y_offset = y_offset

    def update(self):
        """Track ourself relative to the balloon guy"""
        bgr = self.balloon_guy.rect
        self.rect.x = bgr.x + self.x_offset
        self.rect.y = bgr.y + self.y_offset

    
class BalloonGuy(pygame.sprite.Sprite):
    """the balloon guy in the middle of the screen"""
    def __init__(self):
        pygame.sprite.Sprite.__init__(self) #call Sprite initializer
        self.image, self.rect = utilities.load_image('smallest.gif', -1)

    def update(self):
        """Track with the mouse for now"""
        pos = pygame.mouse.get_pos()
        self.rect.midtop = pos

def move_up_handler(up=False):
    # Handle non-interactive movements
    if up:
        background.y_speed += UP_SPEED_INC
        if background.y_speed > MAX_UP_SPEED:
            background.y_speed = MAX_UP_SPEED
    if not up:
        background.y_speed -= UP_SPEED_DEC
        if background.y_speed < 0:
            background.y_speed = 0
    birds.y_speed = background.y_speed
    background.background_handler()
    # Handle interactive movements here?
    
def collision_handler():
    global balloon_count
    for balloon in balloon_list:
        hit = pygame.sprite.spritecollideany(balloon, birds.allsprites)
        if hit:
            allsprites.remove(balloon)
            balloon_list.remove(balloon)
            balloon_count -= 1
            if balloon_count == 0:
                print "game over!"
                sys.exit(0)
            
def main_loop():
    clock = pygame.time.Clock()
    going_up = False
    while 1:
        clock.tick(60)

        for event in pygame.event.get():
            if event.type == QUIT:
                return
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                return
            elif event.type == KEYDOWN:
                going_up = True
            elif event.type == KEYUP:
                going_up = False

        move_up_handler(going_up)
        birds.bird_handler()
            
        collision_handler()
        allsprites.update()
        
        # Screen redrawing
        screen.blit(background.bg, (0, 0))
        background.allsprites.draw(screen)
        birds.allsprites.draw(screen)
        allsprites.draw(screen)
        pygame.display.flip()

def initialize():
    global allsprites, screen, bg, balloon_list, balloon_count
    pygame.init()
    window = pygame.display.set_mode((640, 480))
    pygame.display.set_caption('Hello Pyweek!')
    screen = pygame.display.get_surface()

    background.init(screen)
    birds.init(screen)
    
    balloon_guy = BalloonGuy()
    balloon1 = Balloon(balloon_guy, 0, -64)
    balloon2 = Balloon(balloon_guy, -32, -46)
    balloon3 = Balloon(balloon_guy, 32, -46)
    balloon_list = [balloon1, balloon2, balloon3]
    balloon_count = 3

    allsprites = pygame.sprite.RenderPlain((balloon_guy, balloon1, balloon2, balloon3))
    
def main():
    initialize()

    main_loop()
